@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 
-# Copyright (C) 2012
+# Copyright (C) 2012, 2013, 2014
 # Martin Lambers <marlam@marlam.de>
 #
 # Copying and distribution of this file, with or without modification, are
@@ -15,8 +15,7 @@ $GTA create -d 6,8 -c uint8 -v 42 "$TMPD"/a.gta
 
 $GTA to-mat "$TMPD"/a.gta "$TMPD"/b.mat
 $GTA to-mat "$TMPD"/c.mat < "$TMPD"/a.gta
-# Cannot compare MAT files because libmatio stores the creation time in them.
-#cmp "$TMPD"/b.mat "$TMPD"/c.mat
+cmp "$TMPD"/b.mat "$TMPD"/c.mat
 
 $GTA from-mat "$TMPD"/b.mat "$TMPD"/b.gta
 $GTA from-mat "$TMPD"/c.mat > "$TMPD"/c.gta
