@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 
-# Copyright (C) 2011
+# Copyright (C) 2011, 2013
 # Martin Lambers <marlam@marlam.de>
 #
 # Copying and distribution of this file, with or without modification, are
@@ -26,4 +26,14 @@ cmp "$TMPD"/a.gta "$TMPD"/xxa.gta
 cmp "$TMPD"/a.gta "$TMPD"/xxxa.gta
 cmp "$TMPD"/d.gta "$TMPD"/xd.gta
 
+$GTA create -d 10,10 > "$TMPD"/empty0.gta
+$GTA create -c uint8,uint16,uint32 -n5 > "$TMPD"/empty1.gta
+$GTA create -c uint16 -n5 > "$TMPD"/empty2.gta
+$GTA component-extract -d 0,1 "$TMPD"/a.gta > "$TMPD"/xempty0.gta
+cmp "$TMPD"/empty0.gta "$TMPD"/xempty0.gta
+$GTA component-extract "$TMPD"/empty0.gta > "$TMPD"/xxempty0.gta
+cmp "$TMPD"/empty0.gta "$TMPD"/xxempty0.gta
+$GTA component-extract -k 1 "$TMPD"/empty1.gta > "$TMPD"/xempty2.gta
+cmp "$TMPD"/empty2.gta "$TMPD"/xempty2.gta
+
 rm -r "$TMPD"
